		Title  "Switch Counter Program B"

		list p=16f84 		; processor type
;
; This program is used to control a 7 segment display
; The display is connected to PortB
; The program counts from 0 to 9 and then repeats
; each time a switch connected to RA4 is pressed
;
PCL		equ 0x02		; PCL RAM address
PortA		equ 0x05		; PortA RAM address
Switch		equ 0x04		; Switch = PortA pin RA4
PortB		equ 0x06		; PortB RAM address
TrisA		equ 0x85		; TRISA RAM address
TrisB		equ 0x86		; TRISB RAM address
Status		equ 0x03		; Status RAM address
Z		equ 0x02		; Status Z flag = bit 2
RP0		equ 0x05		; Status RP0 bit = bit 5
Pointer		equ 0x0C		; display data pointer
DelayA		equ 0x0D		; delay counter A byte
DelayB		equ 0x0E		; /WD delay counter B byte
; 
; -------------
; PROGRAM START
; -------------
;
		org 0h			; startup address = 0000

		clrf PortB              ; display is blank
		bsf Status,RP0		; set RP0 for RAM page 1
		movlw b'00011111'	; set all PortA as inputs
		movwf TrisA		; the rest are outputs
		clrf TrisB		; all PortB = outputs
		bcf Status,RP0		; set RP0 for RAM page 0

Loop1		clrf Pointer		; reset the pointer to 0
Loop2		btfsc PortA,Switch      ; test the switch state
		goto Loop2		; it is open
		call Delay50		; execute a 50mS delay

Loop3		btfss PortA,Switch	; test the switch state
		goto Loop3		; it is still closed
		call Delay50		; execute a 50mS delay

		movf Pointer,w		; put Pointer into W
		call DataTable          ; put digit value in W
		movwf PortB		; now display new digit

		incf Pointer		; add 1 to Pointer value
		movlw d'10'		; see if Pointer = 10
		xorwf Pointer,w		; if it is, reset it to 0
		btfss Status,Z		; test Zero bit		
		goto Loop2		; Z = 0, do Loop 2
		goto Loop1		; Z = 1, do Loop 1
;
; -------------------------------
; SUBROUTINE: waste time for 50mS
; -------------------------------
;
Delay50	        clrf DelayA		; /R clear DelayA to 0
		movlw 40h		; set DelayB to 40h
		movwf DelayB
Wait1		decfsz DelayA		; subtract 1 from DelayA
		goto Wait1		; if not 0, goto Wait1
		decfsz DelayB		; subtract 1 from DelayB
		goto Wait1		; if not 0, goto Wait1
		return			; finished the delay
;
; -----------------------------------------
; SUBROUTINE: lookup table for display data
; -----------------------------------------
; Bit 0 = RB0 = segment A
; Bit 1 = RB1 = segment B
; Bit 2 = RB2 = segment C
; Bit 3 = RB3 = segment D
; Bit 4 = RB4 = segment E
; Bit 5 = RB5 = segment F
; Bit 6 = RB6 = segment G
; Bit 7 = RB7 = Decimal Point
;
DataTable	addwf PCL          	; add W value to PCL
		retlw b'00111111'	; 0
		retlw b'00000110'       ; 1
		retlw b'01011011'       ; 2    
		retlw b'01001111'       ; 3   
		retlw b'01100110'       ; 4   
		retlw b'01101101'       ; 5   
		retlw b'01111101'       ; 6   
		retlw b'00000111'       ; 7   
		retlw b'01111111'       ; 8   
		retlw b'01101111'       ; 9   


		end
