
	include "p16f84.inc"

	CBLOCK 0x20
Flags			; register to hold key flag bit
Debounce			; debounce counter
KeyVal			; value of key press
Columns			; columns monitor
Rows			; rows monitor
Count			; delay counter

	ENDC

Key	equ 0h			; flag for keypress

	org 0000h

	clrf PORTA
	clrf PORTB
	bsf STATUS,RP0		; ram page 1
	clrf TRISA		; portA = all outputs
	movlw b'11110111'		; PortB 7 - 4 = inputs
	movwf TRISB		;       3 - 0 = outputs
	movlw b'11000010'		; set TMR0 internal clk
	movwf OPTION_REG		; prescale 1:256
	bcf STATUS,RP0		; ram page 0

	clrf Debounce		; these registers need to be
	clrf Flags		; initialised to zero

MnLoop	btfss INTCON,T0IF		; wait for TMR0 overflow
	goto MnLoop		; approx 65mS @ 4MHz clock

          bcf INTCON,T0IF

	movf Debounce		; if Debounce = 0, test keypad
	btfsc STATUS,Z
	goto TestKP

	decf Debounce		; else decrement it
	goto MnLoop		; and ignore keypad
;
; TEST FOR A KEYPRESS ON THE KEYPAD 
;
TestKP	call KeyCheck		; check keypad

	btfss KeyVal,7		; if = 1 then no key pressed
	goto KeyIsDown

	btfss Flags,Key		; no key pressed
	goto MnLoop		; and key is already released

	bcf Flags,Key		; flag key is released
	movlw 3h			; set debounce period
	movwf Debounce
	goto MnLoop

KeyIsDown	btfsc Flags,Key		; if = 0 then key just pressed
	goto MnLoop		; key is still pressed

	bsf Flags,Key		; flag key is pressed
	movlw 3h			; set debounce period
	movwf Debounce

	movf KeyVal,W		; service key
	movwf PORTA		; write KeyVal to PORTA

	goto MnLoop
;
; SUBROUTINE:
; Check the keypad for a keypress
; Enable each of the 4 rows one at a time (logic high)
; test each of the 4 columns to see if any are logic high
; If so, a key is pressed
;
; ON RETURN:
; KeyVal = value of keypress, bit 7 will be 0 (0xxx xxxx)
; If no keys are pressed, bit 7 of KeyVal = 1 (1xxx xxxx)
;
KeyCheck	movlw b'00000001'		; make ready to set Row 1 high (RB0)
	movwf Rows
	clrf KeyVal		; initialise Key Value

RowLoop	movf Rows,W		; set a Row high
	movwf PORTB

	movlw 4h			; small delay while PORTB stabilizes
	movwf Count
DLoop	decfsz Count
	goto DLoop

	movf PORTB,W		; read portB
	andlw b'11110000'
	movwf Columns		; save the data - only interested in upper 4 bits
	btfss STATUS,Z
	goto ChkColm		; Z = clear, key must be pressed

	movlw 4h			; no bits set for this row
	addwf KeyVal		; looking at next row
	goto NextRow

ChkColm	bcf STATUS,C		; clear the carry bit
	rlf Columns		; if Carry = 1, key is pressed
	btfsc STATUS,C
	goto LowRow		; key is pressed, KeyVal = key

	incf KeyVal		; on next column so increment Key Value
	movf Columns		; if = 0, then all columns checked for this row
	btfss STATUS,Z
	goto ChkColm		; check value of next column

NextRow	bcf STATUS,C		; clear the carry bit
	rlf Rows			; make ready to set next Row = high
	btfss Rows,4		; test if all Rows have been set
	goto RowLoop		; not yet, bit 4 will be set when all rows checked

	movlw 0xFF		; no keys pressed
	movwf KeyVal
LowRow	clrf PORTB		; all rows = low
	return			; no key was pressed, KeyVal bit 7 still = 1

	
	end
